@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2023 Apple Inc. All rights reserved.
  * Copyright (C) 2008, 2010 Nokia Corporation and/or its subsidiary(-ies)
  * Copyright (C) 2012, Samsung Electronics. All rights reserved.
  *
@@ -23,16 +23,15 @@
 #include "Chrome.h"
 
 #include "AppHighlight.h"
+#include "BarcodeDetectorInterface.h"
 #include "ChromeClient.h"
 #include "ContactInfo.h"
 #include "ContactsRequestData.h"
-#include "DOMWindow.h"
 #include "Document.h"
 #include "DocumentType.h"
+#include "FaceDetectorInterface.h"
 #include "FileList.h"
 #include "FloatRect.h"
-#include "Frame.h"
-#include "FrameLoaderClient.h"
 #include "FrameTree.h"
 #include "Geolocation.h"
 #include "HTMLFormElement.h"
@@ -41,15 +40,21 @@
 #include "HitTestResult.h"
 #include "Icon.h"
 #include "InspectorInstrumentation.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "Page.h"
 #include "PageGroupLoadDeferrer.h"
 #include "PopupOpeningObserver.h"
 #include "RenderObject.h"
-#include "ResourceHandle.h"
 #include "Settings.h"
 #include "ShareData.h"
 #include "StorageNamespace.h"
+#include "StorageNamespaceProvider.h"
+#include "TextDetectorInterface.h"
+#include "WebGPU.h"
 #include "WindowFeatures.h"
+#include "WorkerClient.h"
 #include <JavaScriptCore/VM.h>
 #include <wtf/SetForScope.h>
 #include <wtf/Vector.h>
@@ -70,148 +75,146 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
-Chrome::Chrome(Page& page, ChromeClient& client)
+Chrome::Chrome(Page& page, UniqueRef<ChromeClient>&& client)
     : m_page(page)
-    , m_client(client)
+    , m_client(WTFMove(client))
 {
 }
 
 Chrome::~Chrome()
 {
-    m_client.chromeDestroyed();
+    m_client->chromeDestroyed();
 }
 
 void Chrome::invalidateRootView(const IntRect& updateRect)
 {
-    m_client.invalidateRootView(updateRect);
+    m_client->invalidateRootView(updateRect);
 }
 
 void Chrome::invalidateContentsAndRootView(const IntRect& updateRect)
 {
-    m_client.invalidateContentsAndRootView(updateRect);
+    m_client->invalidateContentsAndRootView(updateRect);
 }
 
 void Chrome::invalidateContentsForSlowScroll(const IntRect& updateRect)
 {
-    m_client.invalidateContentsForSlowScroll(updateRect);
+    m_client->invalidateContentsForSlowScroll(updateRect);
 }
 
 void Chrome::scroll(const IntSize& scrollDelta, const IntRect& rectToScroll, const IntRect& clipRect)
 {
-    m_client.scroll(scrollDelta, rectToScroll, clipRect);
+    m_client->scroll(scrollDelta, rectToScroll, clipRect);
     InspectorInstrumentation::didScroll(m_page);
 }
 
 IntPoint Chrome::screenToRootView(const IntPoint& point) const
 {
-    return m_client.screenToRootView(point);
+    return m_client->screenToRootView(point);
 }
 
 IntRect Chrome::rootViewToScreen(const IntRect& rect) const
 {
-    return m_client.rootViewToScreen(rect);
+    return m_client->rootViewToScreen(rect);
 }
     
 IntPoint Chrome::accessibilityScreenToRootView(const IntPoint& point) const
 {
-    return m_client.accessibilityScreenToRootView(point);
+    return m_client->accessibilityScreenToRootView(point);
 }
 
 IntRect Chrome::rootViewToAccessibilityScreen(const IntRect& rect) const
 {
-    return m_client.rootViewToAccessibilityScreen(rect);
+    return m_client->rootViewToAccessibilityScreen(rect);
 }
 
 PlatformPageClient Chrome::platformPageClient() const
 {
-    return m_client.platformPageClient();
+    return m_client->platformPageClient();
 }
 
-void Chrome::contentsSizeChanged(Frame& frame, const IntSize& size) const
+void Chrome::contentsSizeChanged(LocalFrame& frame, const IntSize& size) const
 {
-    m_client.contentsSizeChanged(frame, size);
+    m_client->contentsSizeChanged(frame, size);
 }
 
 void Chrome::scrollContainingScrollViewsToRevealRect(const IntRect& rect) const
 {
-    m_client.scrollContainingScrollViewsToRevealRect(rect);
+    m_client->scrollContainingScrollViewsToRevealRect(rect);
 }
 
 void Chrome::scrollMainFrameToRevealRect(const IntRect& rect) const
 {
-    m_client.scrollMainFrameToRevealRect(rect);
+    m_client->scrollMainFrameToRevealRect(rect);
 }
 
-void Chrome::setWindowRect(const FloatRect& rect) const
+void Chrome::setWindowRect(const FloatRect& rect)
 {
-    m_client.setWindowRect(rect);
+    m_client->setWindowRect(rect);
 }
 
 FloatRect Chrome::windowRect() const
 {
-    return m_client.windowRect();
+    return m_client->windowRect();
 }
 
 FloatRect Chrome::pageRect() const
 {
-    return m_client.pageRect();
+    return m_client->pageRect();
 }
 
-void Chrome::focus() const
+void Chrome::focus()
 {
-    m_client.focus();
+    m_client->focus();
 }
 
-void Chrome::unfocus() const
+void Chrome::unfocus()
 {
-    m_client.unfocus();
+    m_client->unfocus();
 }
 
 bool Chrome::canTakeFocus(FocusDirection direction) const
 {
-    return m_client.canTakeFocus(direction);
+    return m_client->canTakeFocus(direction);
 }
 
-void Chrome::takeFocus(FocusDirection direction) const
+void Chrome::takeFocus(FocusDirection direction)
 {
-    m_client.takeFocus(direction);
+    m_client->takeFocus(direction);
 }
 
-void Chrome::focusedElementChanged(Element* element) const
+void Chrome::focusedElementChanged(Element* element)
 {
-    m_client.focusedElementChanged(element);
+    m_client->focusedElementChanged(element);
 }
 
-void Chrome::focusedFrameChanged(Frame* frame) const
+void Chrome::focusedFrameChanged(LocalFrame* frame)
 {
-    m_client.focusedFrameChanged(frame);
+    m_client->focusedFrameChanged(frame);
 }
 
-Page* Chrome::createWindow(Frame& frame, const WindowFeatures& features, const NavigationAction& action) const
+Page* Chrome::createWindow(LocalFrame& frame, const WindowFeatures& features, const NavigationAction& action)
 {
-    Page* newPage = m_client.createWindow(frame, features, action);
+    Page* newPage = m_client->createWindow(frame, features, action);
     if (!newPage)
         return nullptr;
 
-    if (!features.noopener && !features.noreferrer) {
-        if (auto* oldSessionStorage = m_page.sessionStorage(false))
-            newPage->setSessionStorage(oldSessionStorage->copy(*newPage));
-    }
+    if (!features.noopener && !features.noreferrer)
+        m_page.storageNamespaceProvider().copySessionStorageNamespace(m_page, *newPage);
 
     return newPage;
 }
 
-void Chrome::show() const
+void Chrome::show()
 {
-    m_client.show();
+    m_client->show();
 }
 
 bool Chrome::canRunModal() const
 {
-    return m_client.canRunModal();
+    return m_client->canRunModal();
 }
 
-void Chrome::runModal() const
+void Chrome::runModal()
 {
     // Defer callbacks in all the other pages in this group, so we don't try to run JavaScript
     // in a way that could interact with this view.
@@ -219,77 +222,82 @@ void Chrome::runModal() const
 
     // JavaScript that runs within the nested event loop must not be run in the context of the
     // script that called showModalDialog. Null out entryScope to break the connection.
-    SetForScope entryScopeNullifier { m_page.mainFrame().document()->vm().entryScope, nullptr };
+
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
+    if (!localMainFrame)
+        return;
+
+    SetForScope entryScopeNullifier { localMainFrame->document()->vm().entryScope, nullptr };
 
     TimerBase::fireTimersInNestedEventLoop();
-    m_client.runModal();
+    m_client->runModal();
 }
 
-void Chrome::setToolbarsVisible(bool b) const
+void Chrome::setToolbarsVisible(bool b)
 {
-    m_client.setToolbarsVisible(b);
+    m_client->setToolbarsVisible(b);
 }
 
 bool Chrome::toolbarsVisible() const
 {
-    return m_client.toolbarsVisible();
+    return m_client->toolbarsVisible();
 }
 
-void Chrome::setStatusbarVisible(bool b) const
+void Chrome::setStatusbarVisible(bool b)
 {
-    m_client.setStatusbarVisible(b);
+    m_client->setStatusbarVisible(b);
 }
 
 bool Chrome::statusbarVisible() const
 {
-    return m_client.statusbarVisible();
+    return m_client->statusbarVisible();
 }
 
-void Chrome::setScrollbarsVisible(bool b) const
+void Chrome::setScrollbarsVisible(bool b)
 {
-    m_client.setScrollbarsVisible(b);
+    m_client->setScrollbarsVisible(b);
 }
 
 bool Chrome::scrollbarsVisible() const
 {
-    return m_client.scrollbarsVisible();
+    return m_client->scrollbarsVisible();
 }
 
-void Chrome::setMenubarVisible(bool b) const
+void Chrome::setMenubarVisible(bool b)
 {
-    m_client.setMenubarVisible(b);
+    m_client->setMenubarVisible(b);
 }
 
 bool Chrome::menubarVisible() const
 {
-    return m_client.menubarVisible();
+    return m_client->menubarVisible();
 }
 
-void Chrome::setResizable(bool b) const
+void Chrome::setResizable(bool b)
 {
-    m_client.setResizable(b);
+    m_client->setResizable(b);
 }
 
 bool Chrome::canRunBeforeUnloadConfirmPanel()
 {
-    return m_client.canRunBeforeUnloadConfirmPanel();
+    return m_client->canRunBeforeUnloadConfirmPanel();
 }
 
-bool Chrome::runBeforeUnloadConfirmPanel(const String& message, Frame& frame)
+bool Chrome::runBeforeUnloadConfirmPanel(const String& message, LocalFrame& frame)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
     PageGroupLoadDeferrer deferrer(m_page, true);
 
-    return m_client.runBeforeUnloadConfirmPanel(message, frame);
+    return m_client->runBeforeUnloadConfirmPanel(message, frame);
 }
 
 void Chrome::closeWindow()
 {
-    m_client.closeWindow();
+    m_client->closeWindow();
 }
 
-void Chrome::runJavaScriptAlert(Frame& frame, const String& message)
+void Chrome::runJavaScriptAlert(LocalFrame& frame, const String& message)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
@@ -298,20 +306,20 @@ void Chrome::runJavaScriptAlert(Frame& frame, const String& message)
     notifyPopupOpeningObservers();
     String displayMessage = frame.displayStringModifiedByEncoding(message);
 
-    m_client.runJavaScriptAlert(frame, displayMessage);
+    m_client->runJavaScriptAlert(frame, displayMessage);
 }
 
-bool Chrome::runJavaScriptConfirm(Frame& frame, const String& message)
+bool Chrome::runJavaScriptConfirm(LocalFrame& frame, const String& message)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
     PageGroupLoadDeferrer deferrer(m_page, true);
 
     notifyPopupOpeningObservers();
-    return m_client.runJavaScriptConfirm(frame, frame.displayStringModifiedByEncoding(message));
+    return m_client->runJavaScriptConfirm(frame, frame.displayStringModifiedByEncoding(message));
 }
 
-bool Chrome::runJavaScriptPrompt(Frame& frame, const String& prompt, const String& defaultValue, String& result)
+bool Chrome::runJavaScriptPrompt(LocalFrame& frame, const String& prompt, const String& defaultValue, String& result)
 {
     // Defer loads in case the client method runs a new event loop that would
     // otherwise cause the load to continue while we're in the middle of executing JavaScript.
@@ -320,29 +328,33 @@ bool Chrome::runJavaScriptPrompt(Frame& frame, const String& prompt, const Strin
     notifyPopupOpeningObservers();
     String displayPrompt = frame.displayStringModifiedByEncoding(prompt);
 
-    bool ok = m_client.runJavaScriptPrompt(frame, displayPrompt, frame.displayStringModifiedByEncoding(defaultValue), result);
+    bool ok = m_client->runJavaScriptPrompt(frame, displayPrompt, frame.displayStringModifiedByEncoding(defaultValue), result);
     if (ok)
         result = frame.displayStringModifiedByEncoding(result);
 
     return ok;
 }
 
-void Chrome::setStatusbarText(Frame& frame, const String& status)
+void Chrome::setStatusbarText(LocalFrame& frame, const String& status)
 {
-    m_client.setStatusbarText(frame.displayStringModifiedByEncoding(status));
+    m_client->setStatusbarText(frame.displayStringModifiedByEncoding(status));
 }
 
-void Chrome::mouseDidMoveOverElement(const HitTestResult& result, unsigned modifierFlags)
+void Chrome::mouseDidMoveOverElement(const HitTestResult& result, OptionSet<PlatformEventModifier> modifiers)
 {
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
+    if (!localMainFrame)
+        return;
+
     if (result.innerNode() && result.innerNode()->document().isDNSPrefetchEnabled())
-        m_page.mainFrame().loader().client().prefetchDNS(result.absoluteLinkURL().host().toString());
+        localMainFrame->loader().client().prefetchDNS(result.absoluteLinkURL().host().toString());
 
     String toolTip;
     TextDirection toolTipDirection;
     getToolTip(result, toolTip, toolTipDirection);
-    m_client.mouseDidMoveOverElement(result, modifierFlags, toolTip, toolTipDirection);
+    m_client->mouseDidMoveOverElement(result, modifiers, toolTip, toolTipDirection);
 
-    InspectorInstrumentation::mouseDidMoveOverElement(m_page, result, modifierFlags);
+    InspectorInstrumentation::mouseDidMoveOverElement(m_page, result, modifiers);
 }
 
 void Chrome::getToolTip(const HitTestResult& result, String& toolTip, TextDirection& toolTipDirection)
@@ -401,7 +413,7 @@ void Chrome::getToolTip(const HitTestResult& result, String& toolTip, TextDirect
     }
 }
 
-bool Chrome::print(Frame& frame)
+bool Chrome::print(LocalFrame& frame)
 {
     // FIXME: This should have PageGroupLoadDeferrer, like runModal() or runJavaScriptAlert(), because it's no different from those.
 
@@ -410,18 +422,18 @@ bool Chrome::print(Frame& frame)
         return false;
     }
 
-    m_client.print(frame, frame.document()->titleWithDirection());
+    m_client->print(frame, frame.document()->titleWithDirection());
     return true;
 }
 
 void Chrome::enableSuddenTermination()
 {
-    m_client.enableSuddenTermination();
+    m_client->enableSuddenTermination();
 }
 
 void Chrome::disableSuddenTermination()
 {
-    m_client.disableSuddenTermination();
+    m_client->disableSuddenTermination();
 }
 
 #if ENABLE(INPUT_TYPE_COLOR)
@@ -434,7 +446,7 @@ std::unique_ptr<ColorChooser> Chrome::createColorChooser(ColorChooserClient& cli
     return nullptr;
 #else
     notifyPopupOpeningObservers();
-    return m_client.createColorChooser(client, initialColor);
+    return m_client->createColorChooser(client, initialColor);
 #endif
 }
 
@@ -445,7 +457,7 @@ std::unique_ptr<ColorChooser> Chrome::createColorChooser(ColorChooserClient& cli
 std::unique_ptr<DataListSuggestionPicker> Chrome::createDataListSuggestionPicker(DataListSuggestionsClient& client)
 {
     notifyPopupOpeningObservers();
-    return m_client.createDataListSuggestionPicker(client);
+    return m_client->createDataListSuggestionPicker(client);
 }
 
 #endif
@@ -459,51 +471,51 @@ std::unique_ptr<DateTimeChooser> Chrome::createDateTimeChooser(DateTimeChooserCl
     return nullptr;
 #else
     notifyPopupOpeningObservers();
-    return m_client.createDateTimeChooser(client);
+    return m_client->createDateTimeChooser(client);
 #endif
 }
 
 #endif
 
-void Chrome::runOpenPanel(Frame& frame, FileChooser& fileChooser)
+void Chrome::runOpenPanel(LocalFrame& frame, FileChooser& fileChooser)
 {
     notifyPopupOpeningObservers();
-    m_client.runOpenPanel(frame, fileChooser);
+    m_client->runOpenPanel(frame, fileChooser);
 }
 
 void Chrome::showShareSheet(ShareDataWithParsedURL& shareData, CompletionHandler<void(bool)>&& callback)
 {
-    m_client.showShareSheet(shareData, WTFMove(callback));
+    m_client->showShareSheet(shareData, WTFMove(callback));
 }
 
 void Chrome::showContactPicker(const ContactsRequestData& requestData, CompletionHandler<void(std::optional<Vector<ContactInfo>>&&)>&& callback)
 {
-    m_client.showContactPicker(requestData, WTFMove(callback));
+    m_client->showContactPicker(requestData, WTFMove(callback));
 }
 
 void Chrome::loadIconForFiles(const Vector<String>& filenames, FileIconLoader& loader)
 {
-    m_client.loadIconForFiles(filenames, loader);
+    m_client->loadIconForFiles(filenames, loader);
 }
 
 FloatSize Chrome::screenSize() const
 {
-    return m_client.screenSize();
+    return m_client->screenSize();
 }
 
 FloatSize Chrome::availableScreenSize() const
 {
-    return m_client.availableScreenSize();
+    return m_client->availableScreenSize();
 }
 
 FloatSize Chrome::overrideScreenSize() const
 {
-    return m_client.overrideScreenSize();
+    return m_client->overrideScreenSize();
 }
 
 void Chrome::dispatchDisabledAdaptationsDidChange(const OptionSet<DisabledAdaptations>& disabledAdaptations) const
 {
-    m_client.dispatchDisabledAdaptationsDidChange(disabledAdaptations);
+    m_client->dispatchDisabledAdaptationsDidChange(disabledAdaptations);
 }
 
 void Chrome::dispatchViewportPropertiesDidChange(const ViewportArguments& arguments) const
@@ -512,41 +524,71 @@ void Chrome::dispatchViewportPropertiesDidChange(const ViewportArguments& argume
     if (m_isDispatchViewportDataDidChangeSuppressed)
         return;
 #endif
-    m_client.dispatchViewportPropertiesDidChange(arguments);
+    m_client->dispatchViewportPropertiesDidChange(arguments);
 }
 
 #if ENABLE(APP_HIGHLIGHTS)
 void Chrome::storeAppHighlight(AppHighlight&& highlight) const
 {
-    m_client.storeAppHighlight(WTFMove(highlight));
+    m_client->storeAppHighlight(WTFMove(highlight));
 }
 #endif
 
 void Chrome::setCursor(const Cursor& cursor)
 {
-    m_client.setCursor(cursor);
+    m_client->setCursor(cursor);
 }
 
 void Chrome::setCursorHiddenUntilMouseMoves(bool hiddenUntilMouseMoves)
 {
-    m_client.setCursorHiddenUntilMouseMoves(hiddenUntilMouseMoves);
+    m_client->setCursorHiddenUntilMouseMoves(hiddenUntilMouseMoves);
 }
 
 RefPtr<ImageBuffer> Chrome::createImageBuffer(const FloatSize& size, RenderingMode renderingMode, RenderingPurpose purpose, float resolutionScale, const DestinationColorSpace& colorSpace, PixelFormat pixelFormat, bool avoidBackendSizeCheck) const
 {
-    return m_client.createImageBuffer(size, renderingMode, purpose, resolutionScale, colorSpace, pixelFormat, avoidBackendSizeCheck);
+    return m_client->createImageBuffer(size, renderingMode, purpose, resolutionScale, colorSpace, pixelFormat, avoidBackendSizeCheck);
+}
+
+RefPtr<ImageBuffer> Chrome::sinkIntoImageBuffer(std::unique_ptr<SerializedImageBuffer> imageBuffer)
+{
+    return m_client->sinkIntoImageBuffer(WTFMove(imageBuffer));
+}
+
+std::unique_ptr<WorkerClient> Chrome::createWorkerClient(SerialFunctionDispatcher& dispatcher)
+{
+    return m_client->createWorkerClient(dispatcher);
 }
 
 #if ENABLE(WEBGL)
 RefPtr<GraphicsContextGL> Chrome::createGraphicsContextGL(const GraphicsContextGLAttributes& attributes) const
 {
-    return m_client.createGraphicsContextGL(attributes);
+    return m_client->createGraphicsContextGL(attributes);
 }
 #endif
 
-RefPtr<PAL::WebGPU::GPU> Chrome::createGPUForWebGPU() const
+RefPtr<WebGPU::GPU> Chrome::createGPUForWebGPU() const
 {
-    return m_client.createGPUForWebGPU();
+    return m_client->createGPUForWebGPU();
+}
+
+RefPtr<ShapeDetection::BarcodeDetector> Chrome::createBarcodeDetector(const ShapeDetection::BarcodeDetectorOptions& barcodeDetectorOptions) const
+{
+    return m_client->createBarcodeDetector(barcodeDetectorOptions);
+}
+
+void Chrome::getBarcodeDetectorSupportedFormats(CompletionHandler<void(Vector<ShapeDetection::BarcodeFormat>&&)>&& completionHandler) const
+{
+    return m_client->getBarcodeDetectorSupportedFormats(WTFMove(completionHandler));
+}
+
+RefPtr<ShapeDetection::FaceDetector> Chrome::createFaceDetector(const ShapeDetection::FaceDetectorOptions& faceDetectorOptions) const
+{
+    return m_client->createFaceDetector(faceDetectorOptions);
+}
+
+RefPtr<ShapeDetection::TextDetector> Chrome::createTextDetector() const
+{
+    return m_client->createTextDetector();
 }
 
 PlatformDisplayID Chrome::displayID() const
@@ -561,32 +603,32 @@ void Chrome::windowScreenDidChange(PlatformDisplayID displayID, std::optional<Fr
 
 bool Chrome::selectItemWritingDirectionIsNatural()
 {
-    return m_client.selectItemWritingDirectionIsNatural();
+    return m_client->selectItemWritingDirectionIsNatural();
 }
 
 bool Chrome::selectItemAlignmentFollowsMenuWritingDirection()
 {
-    return m_client.selectItemAlignmentFollowsMenuWritingDirection();
+    return m_client->selectItemAlignmentFollowsMenuWritingDirection();
 }
 
 RefPtr<PopupMenu> Chrome::createPopupMenu(PopupMenuClient& client) const
 {
     notifyPopupOpeningObservers();
-    return m_client.createPopupMenu(client);
+    return m_client->createPopupMenu(client);
 }
 
 RefPtr<SearchPopupMenu> Chrome::createSearchPopupMenu(PopupMenuClient& client) const
 {
     notifyPopupOpeningObservers();
-    return m_client.createSearchPopupMenu(client);
+    return m_client->createSearchPopupMenu(client);
 }
 
 bool Chrome::requiresFullscreenForVideoPlayback()
 {
-    return m_client.requiresFullscreenForVideoPlayback();
+    return m_client->requiresFullscreenForVideoPlayback();
 }
 
-void Chrome::didReceiveDocType(Frame& frame)
+void Chrome::didReceiveDocType(LocalFrame& frame)
 {
 #if !PLATFORM(IOS_FAMILY)
     UNUSED_PARAM(frame);
@@ -595,7 +637,7 @@ void Chrome::didReceiveDocType(Frame& frame)
         return;
 
     auto* doctype = frame.document()->doctype();
-    m_client.didReceiveMobileDocType(doctype && doctype->publicId().containsIgnoringASCIICase("xhtml mobile"_s));
+    m_client->didReceiveMobileDocType(doctype && doctype->publicId().containsIgnoringASCIICase("xhtml mobile"_s));
 #endif
 }
 
