@@ -40,7 +40,7 @@
 #include "WebProcessPool.h"
 #include <WebCore/CompositionUnderline.h>
 #if ENABLE(GAMEPAD)
-#include <WebCore/WPEGamepadProvider.h>
+#include <WebCore/GamepadProviderLibWPE.h>
 #endif
 #include <wpe/wpe.h>
 #include <wtf/NeverDestroyed.h>
@@ -57,7 +57,9 @@ static Vector<View*>& viewsVector()
 
 View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseConfiguration)
     : m_client(makeUnique<API::ViewClient>())
+#if ENABLE(TOUCH_EVENTS)
     , m_touchGestureController(makeUnique<TouchGestureController>())
+#endif
     , m_pageClient(makeUnique<PageClientImpl>(*this))
     , m_size { 800, 600 }
     , m_viewStateFlags { WebCore::ActivityState::WindowIsActive, WebCore::ActivityState::IsFocused, WebCore::ActivityState::IsVisible, WebCore::ActivityState::IsInWindow }
@@ -79,6 +81,11 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
     }
 
     auto* pool = configuration->processPool();
+    if (!pool) {
+        auto processPoolConfiguration = API::ProcessPoolConfiguration::create();
+        pool = &WebProcessPool::create(processPoolConfiguration).leakRef();
+        configuration->setProcessPool(pool);
+    }
     m_pageProxy = pool->createWebPage(*m_pageClient, WTFMove(configuration));
 
 #if ENABLE(MEMORY_SAMPLER)
@@ -103,7 +110,7 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
         [](void* data, uint32_t state)
         {
             auto& view = *reinterpret_cast<View*>(data);
-            OptionSet<WebCore::ActivityState::Flag> flags;
+            OptionSet<WebCore::ActivityState> flags;
             if (state & wpe_view_activity_state_visible)
                 flags.add(WebCore::ActivityState::IsVisible);
             if (state & wpe_view_activity_state_focused) {
@@ -202,7 +209,7 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
                     phase = WebWheelEvent::Phase::PhaseEnded;
 
                 auto& page = view.page();
-                page.handleWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(), phase, momentumPhase));
+                page.handleNativeWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(), phase, momentumPhase));
                 return;
             }
 #endif
@@ -224,12 +231,13 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
 
             if (shouldDispatch) {
                 auto& page = view.page();
-                page.handleWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(), phase, momentumPhase));
+                page.handleNativeWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(), phase, momentumPhase));
             }
         },
         // handle_touch_event
         [](void* data, struct wpe_input_touch_event* event)
         {
+#if ENABLE(TOUCH_EVENTS)
             auto& view = *reinterpret_cast<View*>(data);
             auto& page = view.page();
 
@@ -245,6 +253,7 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
                 WTF::switchOn(generatedEvent,
                     [](TouchGestureController::NoEvent&) { },
                     [](TouchGestureController::ClickEvent&) { },
+                    [](TouchGestureController::ContextMenuEvent&) { },
                     [&](TouchGestureController::AxisEvent& axisEvent)
                     {
 #if WPE_CHECK_VERSION(1, 5, 0)
@@ -253,20 +262,15 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
                         auto* event = &axisEvent.event;
 #endif
                         if (event->type != wpe_input_axis_event_type_null) {
-                            page.handleWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(),
+                            page.handleNativeWheelEvent(WebKit::NativeWebWheelEvent(event, page.deviceScaleFactor(),
                                 axisEvent.phase, WebWheelEvent::Phase::PhaseNone));
                             handledThroughGestureController = true;
                         }
                     });
-
-                // In case of the axis event gesturing, the generic touch event handling should be skipped.
-                // Exception to this are touch-up events that should still be handled just like the corresponding
-                // touch-down events were.
-                if (handledThroughGestureController && event->type != wpe_input_touch_event_type_up)
-                    return;
             }
 
             page.handleTouchEvent(touchEvent);
+#endif
         },
         // padding
         nullptr,
@@ -294,7 +298,7 @@ View::View(struct wpe_view_backend* backend, const API::PageConfiguration& baseC
         [](void* data)
         {
             auto& view = *reinterpret_cast<View*>(data);
-            view.page().fullScreenManager()->requestEnterFullScreen();
+            view.page().fullScreenManager()->requestRestoreFullScreen();
         },
         // request_exit_fullscreen
         [](void* data)
@@ -339,11 +343,6 @@ void View::frameDisplayed()
     m_client->frameDisplayed(*this);
 }
 
-void View::handleDownloadRequest(DownloadProxy& download)
-{
-    m_client->handleDownloadRequest(*this, download);
-}
-
 void View::willStartLoad()
 {
     m_client->willStartLoad(*this);
@@ -359,6 +358,11 @@ void View::didReceiveUserMessage(UserMessage&& message, CompletionHandler<void(U
     m_client->didReceiveUserMessage(*this, WTFMove(message), WTFMove(completionHandler));
 }
 
+WebKitWebResourceLoadManager* View::webResourceLoadManager()
+{
+    return m_client->webResourceLoadManager();
+}
+
 void View::setInputMethodContext(WebKitInputMethodContext* context)
 {
     m_inputMethodFilter.setContext(context);
@@ -377,10 +381,10 @@ void View::setInputMethodState(std::optional<InputMethodState>&& state)
 void View::selectionDidChange()
 {
     const auto& editorState = m_pageProxy->editorState();
-    if (!editorState.isMissingPostLayoutData) {
-        m_inputMethodFilter.notifyCursorRect(editorState.postLayoutData().caretRectAtStart);
-        m_inputMethodFilter.notifySurrounding(editorState.postLayoutData().surroundingContext, editorState.postLayoutData().surroundingContextCursorPosition,
-            editorState.postLayoutData().surroundingContextSelectionPosition);
+    if (editorState.hasPostLayoutAndVisualData()) {
+        m_inputMethodFilter.notifyCursorRect(editorState.visualData->caretRectAtStart);
+        m_inputMethodFilter.notifySurrounding(editorState.postLayoutData->surroundingContext, editorState.postLayoutData->surroundingContextCursorPosition,
+            editorState.postLayoutData->surroundingContextSelectionPosition);
     }
 }
 
@@ -391,7 +395,7 @@ void View::setSize(const WebCore::IntSize& size)
         m_pageProxy->drawingArea()->setSize(size);
 }
 
-void View::setViewState(OptionSet<WebCore::ActivityState::Flag> flags)
+void View::setViewState(OptionSet<WebCore::ActivityState> flags)
 {
     auto changedFlags = m_viewStateFlags ^ flags;
     m_viewStateFlags = flags;
@@ -416,11 +420,17 @@ void View::setViewState(OptionSet<WebCore::ActivityState::Flag> flags)
 
 void View::handleKeyboardEvent(struct wpe_input_keyboard_event* event)
 {
+    auto isAutoRepeat = false;
+    if (event->pressed)
+        isAutoRepeat = m_keyAutoRepeatHandler.keyPress(event->key_code);
+    else
+        m_keyAutoRepeatHandler.keyRelease();
+
     auto filterResult = m_inputMethodFilter.filterKeyEvent(event);
     if (filterResult.handled)
         return;
 
-    page().handleKeyboardEvent(WebKit::NativeWebKeyboardEvent(event, event->pressed ? filterResult.keyText : String(), NativeWebKeyboardEvent::HandledByInputMethod::No, std::nullopt, std::nullopt));
+    page().handleKeyboardEvent(WebKit::NativeWebKeyboardEvent(event, event->pressed ? filterResult.keyText : String(), isAutoRepeat, NativeWebKeyboardEvent::HandledByInputMethod::No, std::nullopt, std::nullopt));
 }
 
 void View::synthesizeCompositionKeyPress(const String& text, std::optional<Vector<WebCore::CompositionUnderline>>&& underlines, std::optional<EditingRange>&& selectionRange)
@@ -430,7 +440,7 @@ void View::synthesizeCompositionKeyPress(const String& text, std::optional<Vecto
     // composition results. WPE doesn't have an equivalent, so we send VoidSymbol
     // here to WebCore. PlatformKeyEvent converts this code into VK_PROCESSKEY.
     static struct wpe_input_keyboard_event event = { 0, WPE_KEY_VoidSymbol, 0, true, 0 };
-    page().handleKeyboardEvent(WebKit::NativeWebKeyboardEvent(&event, text, NativeWebKeyboardEvent::HandledByInputMethod::Yes, WTFMove(underlines), WTFMove(selectionRange)));
+    page().handleKeyboardEvent(WebKit::NativeWebKeyboardEvent(&event, text, false, NativeWebKeyboardEvent::HandledByInputMethod::Yes, WTFMove(underlines), WTFMove(selectionRange)));
 }
 
 void View::close()
@@ -446,6 +456,10 @@ bool View::setFullScreen(bool fullScreenState)
         return false;
 #endif
     m_fullScreenModeActive = fullScreenState;
+    if (m_fullScreenModeActive)
+        m_client->enterFullScreen(*this);
+    else
+        m_client->exitFullScreen(*this);
     return true;
 };
 #endif
@@ -466,7 +480,7 @@ WebKit::WebPageProxy* View::platformWebPageProxyForGamepadInput()
     if (views.isEmpty())
         return nullptr;
 
-    struct wpe_view_backend* viewBackend = WebCore::WPEGamepadProvider::singleton().inputView();
+    struct wpe_view_backend* viewBackend = WebCore::GamepadProviderLibWPE::singleton().inputView();
 
     size_t index = notFound;
 
