@@ -33,8 +33,8 @@ class IntRect;
 }
 
 namespace WebKit {
-class DownloadProxy;
 class WebKitPopupMenu;
+class WebKitWebResourceLoadManager;
 struct WebPopupItem;
 struct UserMessage;
 }
@@ -50,10 +50,15 @@ private:
     bool isGLibBasedAPI() override { return true; }
 
     void frameDisplayed(WKWPE::View&) override;
-    void handleDownloadRequest(WKWPE::View&, WebKit::DownloadProxy&) override;
     void willStartLoad(WKWPE::View&) override;
     void didChangePageID(WKWPE::View&) override;
     void didReceiveUserMessage(WKWPE::View&, WebKit::UserMessage&&, CompletionHandler<void(WebKit::UserMessage&&)>&&) override;
+    WebKit::WebKitWebResourceLoadManager* webResourceLoadManager() override;
+
+#if ENABLE(FULLSCREEN_API)
+    void enterFullScreen(WKWPE::View&) override;
+    void exitFullScreen(WKWPE::View&) override;
+#endif
 
     WebKitWebView* m_webView;
 };
