@@ -27,17 +27,17 @@
 #include "PlatformWebView.h"
 
 #include <WPEToolingBackends/HeadlessViewBackend.h>
+#include <WebKit/WKPagePrivateWPE.h>
 #include <WebKit/WKRetainPtr.h>
 #include <WebKit/WKView.h>
 
 namespace TestWebKitAPI {
 
-PlatformWebView::PlatformWebView(WKContextRef contextRef, WKPageGroupRef pageGroupRef)
+PlatformWebView::PlatformWebView(WKContextRef contextRef)
     : m_window(nullptr)
 {
     WKRetainPtr<WKPageConfigurationRef> configuration = adoptWK(WKPageConfigurationCreate());
     WKPageConfigurationSetContext(configuration.get(), contextRef);
-    WKPageConfigurationSetPageGroup(configuration.get(), pageGroupRef);
 
     initialize(configuration.get());
 }
@@ -53,9 +53,12 @@ PlatformWebView::PlatformWebView(WKPageRef relatedPage)
 {
     WKRetainPtr<WKPageConfigurationRef> configuration = adoptWK(WKPageConfigurationCreate());
     WKPageConfigurationSetContext(configuration.get(), WKPageGetContext(relatedPage));
-    WKPageConfigurationSetPageGroup(configuration.get(), WKPageGetPageGroup(relatedPage));
     WKPageConfigurationSetRelatedPage(configuration.get(), relatedPage);
 
+    auto relatedConfiguration = adoptWK(WKPageCopyPageConfiguration(relatedPage));
+    if (auto* preferences = WKPageConfigurationGetPreferences(relatedConfiguration.get()))
+        WKPageConfigurationSetPreferences(configuration.get(), preferences);
+
     initialize(configuration.get());
 }
 
@@ -82,22 +85,32 @@ void PlatformWebView::resizeTo(unsigned width, unsigned height)
 
 void PlatformWebView::simulateSpacebarKeyPress()
 {
-    // FIXME: implement this.
+    // https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/code
+    WKPageHandleKeyboardEvent(page(), WKKeyboardEventMake(kWKEventKeyDown, kWKInputTypeNormal, " ", 1, WPE_KEY_space, 0x0041, 0));
+    WKPageHandleKeyboardEvent(page(), WKKeyboardEventMake(kWKEventKeyUp, kWKInputTypeNormal, " ", 1, WPE_KEY_space, 0x0041, 0));
 }
 
 void PlatformWebView::simulateAltKeyPress()
 {
-    // FIXME: implement this.
+    // https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/code
+    WKPageHandleKeyboardEvent(page(), WKKeyboardEventMake(kWKEventKeyDown, kWKInputTypeNormal, nullptr, 0, WPE_KEY_Alt_L, 0x0040, 0));
+    WKPageHandleKeyboardEvent(page(), WKKeyboardEventMake(kWKEventKeyUp, kWKInputTypeNormal, nullptr, 0, WPE_KEY_Alt_L, 0x0040, 0));
 }
 
 void PlatformWebView::simulateRightClick(unsigned x, unsigned y)
 {
-    // FIXME: implement this.
+    simulateButtonClick(kWKEventMouseButtonRightButton, x, y, 0);
 }
 
-void PlatformWebView::simulateMouseMove(unsigned x, unsigned y, WKEventModifiers)
+void PlatformWebView::simulateButtonClick(WKEventMouseButton button, unsigned x, unsigned y, WKEventModifiers modifiers)
 {
-    // FIXME: implement this.
+    WKPageHandleMouseEvent(page(), WKMouseEventMake(kWKEventMouseDown, button, WKPointMake(x, y), 0, modifiers));
+    WKPageHandleMouseEvent(page(), WKMouseEventMake(kWKEventMouseUp, button, WKPointMake(x, y), 0, modifiers));
+}
+
+void PlatformWebView::simulateMouseMove(unsigned x, unsigned y, WKEventModifiers modifiers)
+{
+    WKPageHandleMouseEvent(page(), WKMouseEventMake(kWKEventMouseMove, kWKEventMouseButtonNoButton, WKPointMake(x, y), 0, modifiers));
 }
 
 } // namespace TestWebKitAPI
